SystemOrganization addCategory: #'XML-Writer-Tests'!

TestCase subclass: #XMLWriterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Tests'!

!XMLWriterTest commentStamp: '<historical>' prior: 0!
A simple test case class. 
Please add more tests to it!!!

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/6/2014 23:34'!
testAttributes
	| writer tag |

	writer := XMLWriter new.
	tag := writer  tag: 'outer' attributes: (Array with: 'first' -> 'one').
	self
		assert: tag hasAttributes;
		assert: (tag attributeAt: 'first') = 'one';
		assert: (tag attributeAt: 'second' put: 'two') = 'two';
		assert: (tag attributeAt: 'third' ifAbsentPut: ['three']) = 'three';
		assert: (tag attributeAt: 'fourth' ifAbsent: [true]);
		assert: (tag attributeAt: 'fourth') isEmpty;
		assert: (tag attributes at: 'fourth') isEmpty;
		deny: (tag includesAttribute: 'fourth').
	tag with: [
		writer
			tag: 'inner'
			attributes: (Dictionary with: 'fourth' -> 'four')
			with: 'test'].
	self assert:
		writer contents =
			'<outer first="one" second="two" third="three"><inner fourth="four">test</inner></outer>'.! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/12/2014 22:55'!
testCDataSections
	| writer |

	(writer := XMLWriter new)
		tag: 'cdata'
		with: [
			writer cdata: '<escaped name="&value;" />'.
			writer cdata with: [
				writer tag
					name: 'escaped';
					with: [writer & 'value']]].

	self assert:
		writer contents =
			('<cdata>',
			'<!![CDATA[<escaped name="&value;" />]]>',
			'<!![CDATA[<escaped>&value;</escaped>]]>',
			'</cdata>').! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/12/2014 22:22'!
testCanonicalDTDs
	| writer |

	(writer := XMLWriter new)
		isCanonical: true;
		comment: 'start of dtd'.
	writer doctype
		root: 'element';
		systemID: 'doc.dtd';
		with: [
			writer comment: 'ignored'.
			writer elementDeclaration
				name: 'ignored';
				contentModel: 'EMPTY'.
			writer entityDeclaration
				name: 'ignored';
				replacement: 'ignored'.
			writer attributeDeclaration
				element: 'ignored';
				attribute: 'ignored';
				type: 'CDATA';
				defaultPragma: '#REQUIRED'.
			writer notationDeclaration
				name: 'external';
				systemID: 'external.txt'.
			writer pi
				target: 'ignored';
				data: 'ignored'].
	writer comment: 'end of dtd'.

	self assert: writer contents = '<!!--start of dtd--><!!--end of dtd-->'.! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/12/2014 23:06'!
testCanonicalTags
	| writer |

	(writer := XMLWriter new)
		isCanonical: true;
		comment: 'tags'.
	(writer tag: 'tags')
		attributeAt: 'c' put: '3';
		attributeAt: 'a' put: '1';
		attributeAt: 'b' put: '2';
		xmlnsAt: 'b' put: 'ns2';
		xmlnsAt: 'a' put: 'ns1';
		xmlns: 'default';
		with: [
			writer
				tag: 'cdata'
				with: [writer cdata: '<escaped name="&value;" />'].
			writer
				tag: 'empty'
				with: nil].

	self assert:
		writer contents =
			('<!!--tags-->',
			'<tags xmlns="default" xmlns:a="ns1" xmlns:b="ns2" a="1" b="2" c="3">',
			'<cdata>&lt;escaped name=&quot;&amp;value;&quot; /&gt;</cdata>',
			'<empty></empty>',
			'</tags>').! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/12/2014 23:03'!
testCanonicalXMLDeclarationsAndPIs
	| writer |

	(writer := XMLWriter new)
		isCanonical: true;
		xml;
		comment: 'start of pi'.
	writer pi
		target: 'pi';
		data: 'preserved'.
	writer comment: 'end of pi'.

	self assert: writer contents = '<!!--start of pi--><?pi preserved?><!!--end of pi-->'.! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 2/9/2014 03:56'!
testContentEscaping
	| writer |

	(writer := XMLWriter new)
		tag
			name: 'test';
			attributeAt: 'escaped' put: '& " < >';
			with: '& " < >'.
	self assert:
		writer contents =
			'<test escaped="&amp; &quot; &lt; &gt;">&amp; &quot; &lt; &gt;</test>'.

	((writer := XMLWriter new)
		isEscaping: false;
		tag)
			name: 'test';
			attributeAt: 'unescaped' put: '& " < >';
			with: '& " < >'.
	self assert: writer contents = '<test unescaped="& " < >">& " < ></test>'.! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/12/2014 07:45'!
testEntityReference
	| writer |

	(writer := XMLWriter new)
		tag: 'entities'
		with: [
			writer & 'one'.
			writer string: 'two'.
			writer & 'three'].

	self assert: writer contents = '<entities>&one;two&three;</entities>'.! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 2/9/2014 03:52'!
testEntityValueEscaping
	| writer |

	(writer := XMLWriter new)
		entityDeclaration
			name: 'test';
			replacement: '% " < >';
			write.
	self assert: writer contents = '<!!ENTITY test "&#37; &#34; < >">'.

	((writer := XMLWriter new)
		isEscaping: false;
		entityDeclaration)
			name: 'test';
			replacement: '% " < >';
			write.
	self assert: writer contents = '<!!ENTITY test "% " < >">'.
! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/12/2014 07:39'!
testPrettyPrintingDTDs
	| writer  |

	(writer := XMLWriter new)
		enablePrettyPrinting;
		xml.
	writer doctype
		root: 'pretty';
		with: [
			writer comment: 'element'.
			writer elementDeclaration
				name: 'pretty';
				contentModel: 'EMPTY'.
			writer comment: 'entity'.
			writer entityDeclaration
				isParameter: true;
				name: 'comment';
				replacement: '<!!--attribute-->'.
			writer % 'comment'.
			writer attributeDeclaration
				element: 'pretty';
				attribute: 'name';
				type: 'CDATA';
				defaultPragma: '#REQUIRED'.
			writer comment: 'notation'.
			writer notationDeclaration
				name: 'external';
				systemID: 'external.txt'].

	self assert: writer contents = '<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE pretty [
    <!!--element-->
    <!!ELEMENT pretty EMPTY>
    <!!--entity-->
    <!!ENTITY % comment "<!!--attribute-->">
    %comment;
    <!!ATTLIST pretty name CDATA #REQUIRED>
    <!!--notation-->
    <!!NOTATION external SYSTEM "external.txt">
]>
'.! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/12/2014 07:40'!
testPrettyPrintingElements
	| writer  |

	(writer := XMLWriter new)
		enablePrettyPrinting;
		xml;
		comment: 'an example'.
	writer tag
		name: 'example:user';
		xmlnsAt: 'example' put: 'urn://example';
		with: [
			writer
				tag: 'username' with: 'jsmith';
				tag: 'real-name' with: 'John Smith';
				tag: 'age' with: '25';
				tag: 'id' with: '101'].

	self assert: writer contents = '<?xml version="1.0" encoding="UTF-8"?>
<!!--an example-->
<example:user xmlns:example="urn://example">
    <username>jsmith</username>
    <real-name>John Smith</real-name>
    <age>25</age>
    <id>101</id>
</example:user>'! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/12/2014 21:57'!
testSelfClosingTags
	| writer |

	(writer := XMLWriter new)
		tag: 'tags' with: [
			writer
				tag: 'self-closing' with: nil;
				outputsSelfClosingTags: false;
				tag: 'not-self-closing' with: nil;
				outputsSelfClosingTags: true;
				tag: 'self-closing' with: nil].

	self assert:
		writer printString =
			'<tags><self-closing /><not-self-closing></not-self-closing><self-closing /></tags>'.! !

!XMLWriterTest methodsFor: 'tests' stamp: 'StephaneDucasse 2/17/2013 10:03'!
testSimpleExample

	| writer  |
	writer := XMLWriter new.
	writer tag
			name: 'emptyTag';
			attributeAt: 'a' put: 'one';
			write.
			
	self assert: writer contents  =  '<emptyTag a="one" />'! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/26/2014 08:08'!
testSimpleExampleWithWriteWith
	| block contents |

	block :=
		[:writer |
			writer tag
				name: 'emptyTag';
				attributeAt: 'a' put: 'value'].
	contents := '<emptyTag a="value" />'.
	self
		assert: (XMLWriter new writeWith: block) contents = contents;
		assert: (XMLWriter writeWith: block) contents = contents.! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/7/2014 00:10'!
testStartingAndFinishing
	| writer outer inner |

	writer := XMLWriter new.

	(outer := writer tag: 'outer')
		attributeAt: 'test' put: 'value';
		startWriting.

	(inner := writer tag: 'inner')
		startWriting.
	writer string: 'test'.
	inner finishWriting.

	self assert: writer contents = '<outer test="value"><inner>test</inner></outer>'.! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 2/9/2014 04:05'!
testSystemAndPublicIDs
	| writer |

	(writer := XMLWriter new)
		doctype
			root: 'test';
			publicID: 'public"ID';
			systemID: 'system"ID'''. "contains double and single quote"

	"IDs are single quoted because they contain a double quote"
	self assert: writer contents = '<!!DOCTYPE test PUBLIC ''public"ID'' ''system"ID''>'! !

!XMLWriterTest methodsFor: 'tests' stamp: 'monty 10/12/2014 23:18'!
testXmlnsDeclarations
	| writer tag |

	writer := XMLWriter new.
	tag := writer  tag: 'test'.
	tag xmlnsDeclarations: (Array with: '' -> 'default1' with: 'ns1' -> 'first').
	self
		assert: tag xmlns = 'default1';
		assert: (tag xmlnsAt: '') = 'default1';
		assert: (tag xmlns: 'default2') = tag;
		assert: tag xmlns = 'default2';
		assert: (tag xmlnsAt: 'ns1') = 'first';
		assert: (tag xmlnsAt: 'ns2' put: 'second') = 'second';
		assert: (tag xmlnsAt: 'ns3' ifAbsentPut: ['third']) = 'third';
		assert: (tag xmlnsAt: 'ns4' ifAbsent: [true]);
		assert: (tag xmlnsAt: 'ns4') isEmpty;
		assert: (tag xmlnsDeclarations at: 'ns4') isEmpty.

	self assert:
		writer contents =
			'<test xmlns="default2" xmlns:ns1="first" xmlns:ns2="second" xmlns:ns3="third" />'.! !
